@@ -13,30 +13,28 @@
 # limitations under the License.
 
 """GridFS implementation for Motor, an asynchronous driver for MongoDB."""
-
+import hashlib
 import warnings
 
 import gridfs
 import pymongo
 import pymongo.errors
-from gridfs import (DEFAULT_CHUNK_SIZE,
-                    grid_file)
+from gridfs import DEFAULT_CHUNK_SIZE, grid_file
 
-from motor.core import (AgnosticCursor,
-                        AgnosticCollection,
-                        AgnosticDatabase)
-from motor.docstrings import *
-from motor.metaprogramming import (AsyncCommand,
-                                   AsyncRead,
-                                   coroutine_annotation,
-                                   create_class_with_framework,
-                                   DelegateMethod,
-                                   MotorCursorChainingMethod,
-                                   ReadOnlyProperty)
+from motor import docstrings
+from motor.core import AgnosticCollection, AgnosticCursor, AgnosticDatabase
+from motor.metaprogramming import (
+    AsyncCommand,
+    AsyncRead,
+    DelegateMethod,
+    ReadOnlyProperty,
+    coroutine_annotation,
+    create_class_with_framework,
+)
 
 
 class AgnosticGridOutCursor(AgnosticCursor):
-    __motor_class_name__ = 'MotorGridOutCursor'
+    __motor_class_name__ = "MotorGridOutCursor"
     __delegate_class__ = gridfs.GridOutCursor
 
     # PyMongo's GridOutCursor inherits __die from Cursor.
@@ -44,29 +42,29 @@ class AgnosticGridOutCursor(AgnosticCursor):
 
     def next_object(self):
         """**DEPRECATED** - Get next GridOut object from cursor."""
+        # Note: the super() call will raise a warning for the deprecation.
         grid_out = super().next_object()
         if grid_out:
             grid_out_class = create_class_with_framework(
-                AgnosticGridOut, self._framework, self.__module__)
+                AgnosticGridOut, self._framework, self.__module__
+            )
 
             return grid_out_class(self.collection, delegate=grid_out)
         else:
             # Exhausted.
             return None
 
-    def _close_exhaust_cursor(self):
-        # Exhaust MotorGridOutCursors are prohibited.
-        pass
-
 
 class MotorGridOutProperty(ReadOnlyProperty):
     """Creates a readonly attribute on the wrapped PyMongo GridOut."""
+
     def create_attribute(self, cls, attr_name):
         def fget(obj):
             if not obj.delegate._file:
                 raise pymongo.errors.InvalidOperation(
                     "You must call MotorGridOut.open() before accessing "
-                    "the %s property" % attr_name)
+                    "the %s property" % attr_name
+                )
 
             return getattr(obj.delegate, attr_name)
 
@@ -74,7 +72,7 @@ class MotorGridOutProperty(ReadOnlyProperty):
         return property(fget=fget, doc=doc)
 
 
-class AgnosticGridOut(object):
+class AgnosticGridOut:
     """Class to read data out of GridFS.
 
     MotorGridOut supports the same attributes as PyMongo's
@@ -86,48 +84,49 @@ class AgnosticGridOut(object):
     instantiated directly, call :meth:`open`, :meth:`read`, or
     :meth:`readline` before accessing its attributes.
     """
-    __motor_class_name__ = 'MotorGridOut'
+
+    __motor_class_name__ = "MotorGridOut"
     __delegate_class__ = gridfs.GridOut
 
     _ensure_file = AsyncCommand()
-    _id          = MotorGridOutProperty()
-    aliases      = MotorGridOutProperty()
-    chunk_size   = MotorGridOutProperty()
-    close        = MotorGridOutProperty()
+    _id = MotorGridOutProperty()
+    aliases = MotorGridOutProperty()
+    chunk_size = MotorGridOutProperty()
+    close = MotorGridOutProperty()
     content_type = MotorGridOutProperty()
-    filename     = MotorGridOutProperty()
-    length       = MotorGridOutProperty()
-    md5          = MotorGridOutProperty()
-    metadata     = MotorGridOutProperty()
-    name         = MotorGridOutProperty()
-    read         = AsyncRead()
-    readable     = DelegateMethod()
-    readchunk    = AsyncRead()
-    readline     = AsyncRead()
-    seek         = DelegateMethod()
-    seekable     = DelegateMethod()
-    tell         = DelegateMethod()
-    upload_date  = MotorGridOutProperty()
-    write        = DelegateMethod()
+    filename = MotorGridOutProperty()
+    length = MotorGridOutProperty()
+    metadata = MotorGridOutProperty()
+    name = MotorGridOutProperty()
+    read = AsyncRead()
+    readable = DelegateMethod()
+    readchunk = AsyncRead()
+    readline = AsyncRead()
+    seek = DelegateMethod()
+    seekable = DelegateMethod()
+    tell = DelegateMethod()
+    upload_date = MotorGridOutProperty()
+    write = DelegateMethod()
 
-    def __init__(self, root_collection, file_id=None, file_document=None,
-                 delegate=None, session=None):
+    def __init__(
+        self, root_collection, file_id=None, file_document=None, delegate=None, session=None
+    ):
         collection_class = create_class_with_framework(
-            AgnosticCollection, self._framework, self.__module__)
+            AgnosticCollection, self._framework, self.__module__
+        )
 
         if not isinstance(root_collection, collection_class):
             raise TypeError(
                 "First argument to MotorGridOut must be "
-                "MotorCollection, not %r" % root_collection)
+                "MotorCollection, not %r" % root_collection
+            )
 
         if delegate:
             self.delegate = delegate
         else:
             self.delegate = self.__delegate_class__(
-                root_collection.delegate,
-                file_id,
-                file_document,
-                session=session)
+                root_collection.delegate, file_id, file_document, session=session
+            )
 
         self.io_loop = root_collection.get_io_loop()
 
@@ -143,8 +142,8 @@ class AgnosticGridOut(object):
     def __getattr__(self, item):
         if not self.delegate._file:
             raise pymongo.errors.InvalidOperation(
-                "You must call MotorGridOut.open() before accessing "
-                "the %s property" % item)
+                "You must call MotorGridOut.open() before accessing the %s property" % item
+            )
 
         return getattr(self.delegate, item)
 
@@ -161,9 +160,7 @@ class AgnosticGridOut(object):
            :class:`~motor.MotorGridOut` now opens itself on demand, calling
            ``open`` explicitly is rarely needed.
         """
-        return self._framework.chain_return_value(self._ensure_file(),
-                                                  self.get_io_loop(),
-                                                  self)
+        return self._framework.chain_return_value(self._ensure_file(), self.get_io_loop(), self)
 
     def get_io_loop(self):
         return self.io_loop
@@ -182,7 +179,7 @@ class AgnosticGridOut(object):
                 @tornado.web.asynchronous
                 @gen.coroutine
                 def get(self, filename):
-                    db = self.settings['db']
+                    db = self.settings["db"]
                     fs = await motor.MotorGridFSBucket(db())
                     try:
                         gridout = await fs.open_download_stream_by_name(filename)
@@ -208,29 +205,31 @@ class AgnosticGridOut(object):
             written += len(chunk)
 
 
-class AgnosticGridIn(object):
-    __motor_class_name__ = 'MotorGridIn'
+class AgnosticGridIn:
+    __motor_class_name__ = "MotorGridIn"
     __delegate_class__ = gridfs.GridIn
 
-    __getattr__  = DelegateMethod()
-    _id          = ReadOnlyProperty()
-    abort        = AsyncCommand()
-    chunk_size   = ReadOnlyProperty()
-    closed       = ReadOnlyProperty()
-    close        = AsyncCommand()
+    __getattr__ = DelegateMethod()
+    _id = ReadOnlyProperty()
+    abort = AsyncCommand()
+    chunk_size = ReadOnlyProperty()
+    closed = ReadOnlyProperty()
+    close = AsyncCommand()
     content_type = ReadOnlyProperty()
-    filename     = ReadOnlyProperty()
-    length       = ReadOnlyProperty()
-    md5          = ReadOnlyProperty()
-    name         = ReadOnlyProperty()
-    read         = DelegateMethod()
-    readable     = DelegateMethod()
-    seekable     = DelegateMethod()
-    upload_date  = ReadOnlyProperty()
-    write        = AsyncCommand().unwrap('MotorGridOut')
-    writeable    = DelegateMethod()
-    writelines   = AsyncCommand().unwrap('MotorGridOut')
-    set          = AsyncCommand(attr_name='__setattr__', doc="""
+    filename = ReadOnlyProperty()
+    length = ReadOnlyProperty()
+    name = ReadOnlyProperty()
+    read = DelegateMethod()
+    readable = DelegateMethod()
+    seekable = DelegateMethod()
+    upload_date = ReadOnlyProperty()
+    write = AsyncCommand().unwrap("MotorGridOut")
+    writeable = DelegateMethod()
+    writelines = AsyncCommand().unwrap("MotorGridOut")
+    _exit = AsyncCommand("__exit__")
+    set = AsyncCommand(
+        attr_name="__setattr__",
+        doc="""
 Set an arbitrary metadata attribute on the file. Stores value on the server
 as a key-value pair within the file document once the file is closed. If
 the file is already closed, calling :meth:`set` will immediately update the file
@@ -243,17 +242,17 @@ Metadata set on the file appears as attributes on a
   - `name`: Name of the attribute, will be stored as a key in the file
     document on the server
   - `value`: Value of the attribute
-""")
+""",
+    )
 
-    def __init__(self, root_collection, delegate=None, session=None,
-                 disable_md5=False, **kwargs):
+    def __init__(self, root_collection, delegate=None, session=None, **kwargs):
         """
         Class to write data to GridFS. Application developers should not
         generally need to instantiate this class - see
         :meth:`~motor.MotorGridFSBucket.open_upload_stream`.
 
         Any of the file level options specified in the `GridFS Spec
-        <http://dochub.mongodb.org/core/gridfs>`_ may be passed as
+        <http://dochub.mongodb.org/core/gridfs/>`_ may be passed as
         keyword arguments. Any additional keyword arguments will be
         set as additional fields on the file document. Valid keyword
         arguments include:
@@ -281,63 +280,72 @@ Metadata set on the file appears as attributes on a
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession` to use for all
             commands
-          - `disable_md5` (optional): When True, an MD5 checksum will not be
-            computed for the uploaded file. Useful in environments where
-            MD5 cannot be used for regulatory or other reasons. Defaults to
-            False.
           - `**kwargs` (optional): file level options (see above)
 
+        .. versionchanged:: 3.0
+           Removed support for the `disable_md5` parameter (to match the
+           GridIn class in PyMongo).
         .. versionchanged:: 0.2
            ``open`` method removed, no longer needed.
         """
         collection_class = create_class_with_framework(
-            AgnosticCollection, self._framework, self.__module__)
+            AgnosticCollection, self._framework, self.__module__
+        )
 
         if not isinstance(root_collection, collection_class):
             raise TypeError(
-                "First argument to MotorGridIn must be "
-                "MotorCollection, not %r" % root_collection)
+                "First argument to MotorGridIn must be MotorCollection, not %r" % root_collection
+            )
 
         self.io_loop = root_collection.get_io_loop()
-        if delegate:
-            # Short cut.
-            self.delegate = delegate
-        else:
-            self.delegate = self.__delegate_class__(
-                root_collection.delegate,
-                session=session,
-                disable_md5=disable_md5,
-                **kwargs)
+        # Short cut.
+        self.delegate = delegate or self.__delegate_class__(
+            root_collection.delegate, session=session, **kwargs
+        )
 
     # Support "async with bucket.open_upload_stream() as f:"
     async def __aenter__(self):
         return self
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
-        await self.close()
+        await self._exit(exc_type, exc_val, exc_tb)
 
     def get_io_loop(self):
         return self.io_loop
 
 
-class AgnosticGridFSBucket(object):
-    __motor_class_name__ = 'MotorGridFSBucket'
+class AgnosticGridFSBucket:
+    __motor_class_name__ = "MotorGridFSBucket"
     __delegate_class__ = gridfs.GridFSBucket
 
-    delete                       = AsyncCommand()
-    download_to_stream           = AsyncCommand()
-    download_to_stream_by_name   = AsyncCommand()
-    open_download_stream         = AsyncCommand().wrap(gridfs.GridOut)
-    open_download_stream_by_name = AsyncCommand().wrap(gridfs.GridOut)
-    open_upload_stream           = DelegateMethod().wrap(gridfs.GridIn)
-    open_upload_stream_with_id   = DelegateMethod().wrap(gridfs.GridIn)
-    rename                       = AsyncCommand()
-    upload_from_stream           = AsyncCommand()
-    upload_from_stream_with_id   = AsyncCommand()
+    delete = AsyncCommand(doc=docstrings.gridfs_delete_doc)
+    download_to_stream = AsyncCommand(doc=docstrings.gridfs_download_to_stream_doc)
+    download_to_stream_by_name = AsyncCommand(doc=docstrings.gridfs_download_to_stream_by_name_doc)
+    open_download_stream = AsyncCommand(doc=docstrings.gridfs_open_download_stream_doc).wrap(
+        gridfs.GridOut
+    )
+    open_download_stream_by_name = AsyncCommand(
+        doc=docstrings.gridfs_open_download_stream_by_name_doc
+    ).wrap(gridfs.GridOut)
+    open_upload_stream = DelegateMethod(doc=docstrings.gridfs_open_upload_stream_doc).wrap(
+        gridfs.GridIn
+    )
+    open_upload_stream_with_id = DelegateMethod(
+        doc=docstrings.gridfs_open_upload_stream_with_id_doc
+    ).wrap(gridfs.GridIn)
+    rename = AsyncCommand(doc=docstrings.gridfs_rename_doc)
+    upload_from_stream = AsyncCommand(doc=docstrings.gridfs_upload_from_stream_doc)
+    upload_from_stream_with_id = AsyncCommand(doc=docstrings.gridfs_upload_from_stream_with_id_doc)
 
-    def __init__(self, database, bucket_name="fs", disable_md5=False,
-                 chunk_size_bytes=DEFAULT_CHUNK_SIZE, write_concern=None,
-                 read_preference=None, collection=None):
+    def __init__(
+        self,
+        database,
+        bucket_name="fs",
+        chunk_size_bytes=DEFAULT_CHUNK_SIZE,
+        write_concern=None,
+        read_preference=None,
+        collection=None,
+    ):
         """Create a handle to a GridFS bucket.
 
         Raises :exc:`~pymongo.errors.ConfigurationError` if `write_concern`
@@ -357,12 +365,12 @@ class AgnosticGridFSBucket(object):
             (the default) db.write_concern is used.
           - `read_preference` (optional): The read preference to use. If
             ``None`` (the default) db.read_preference is used.
-          - `disable_md5` (optional): When True, MD5 checksums will not be
-            computed for uploaded files. Useful in environments where MD5
-            cannot be used for regulatory or other reasons. Defaults to False.
           - `collection` (optional): Deprecated, an alias for `bucket_name`
             that exists solely to provide backwards compatibility.
 
+        .. versionchanged:: 3.0
+           Removed support for the `disable_md5` parameter (to match the
+           GridFSBucket class in PyMongo).
         .. versionchanged:: 2.1
            Added support for the `bucket_name`, `chunk_size_bytes`,
            `write_concern`, and `read_preference` parameters.
@@ -374,31 +382,31 @@ class AgnosticGridFSBucket(object):
         """
         # Preserve backwards compatibility of "collection" parameter
         if collection is not None:
-            warnings.warn('the "collection" parameter is deprecated, use '
-                          '"bucket_name" instead', DeprecationWarning,
-                          stacklevel=2)
+            warnings.warn(
+                'the "collection" parameter is deprecated, use "bucket_name" instead',
+                DeprecationWarning,
+                stacklevel=2,
+            )
             bucket_name = collection
 
-        db_class = create_class_with_framework(
-            AgnosticDatabase, self._framework, self.__module__)
+        db_class = create_class_with_framework(AgnosticDatabase, self._framework, self.__module__)
 
         if not isinstance(database, db_class):
             raise TypeError(
-                "First argument to %s must be  MotorDatabase, not %r" % (
-                    self.__class__, database))
+                f"First argument to {self.__class__} must be  MotorDatabase, not {database!r}"
+            )
 
         self.io_loop = database.get_io_loop()
         self.collection = database.get_collection(
-            bucket_name,
-            write_concern=write_concern,
-            read_preference=read_preference)
+            bucket_name, write_concern=write_concern, read_preference=read_preference
+        )
         self.delegate = self.__delegate_class__(
             database.delegate,
             bucket_name,
             chunk_size_bytes=chunk_size_bytes,
             write_concern=write_concern,
             read_preference=read_preference,
-            disable_md5=disable_md5)
+        )
 
     def get_io_loop(self):
         return self.io_loop
@@ -406,27 +414,24 @@ class AgnosticGridFSBucket(object):
     def wrap(self, obj):
         if obj.__class__ is grid_file.GridIn:
             grid_in_class = create_class_with_framework(
-                AgnosticGridIn, self._framework, self.__module__)
+                AgnosticGridIn, self._framework, self.__module__
+            )
 
-            return grid_in_class(
-                root_collection=self.collection,
-                delegate=obj)
+            return grid_in_class(root_collection=self.collection, delegate=obj)
 
         elif obj.__class__ is grid_file.GridOut:
             grid_out_class = create_class_with_framework(
-                AgnosticGridOut, self._framework, self.__module__)
+                AgnosticGridOut, self._framework, self.__module__
+            )
 
-            return grid_out_class(
-                root_collection=self.collection,
-                delegate=obj)
+            return grid_out_class(root_collection=self.collection, delegate=obj)
 
         elif obj.__class__ is gridfs.GridOutCursor:
             grid_out_class = create_class_with_framework(
-                AgnosticGridOutCursor, self._framework, self.__module__)
+                AgnosticGridOutCursor, self._framework, self.__module__
+            )
 
-            return grid_out_class(
-                cursor=obj,
-                collection=self.collection)
+            return grid_out_class(cursor=obj, collection=self.collection)
 
     def find(self, *args, **kwargs):
         """Find and return the files collection documents that match ``filter``.
@@ -483,6 +488,20 @@ class AgnosticGridFSBucket(object):
         """
         cursor = self.delegate.find(*args, **kwargs)
         grid_out_cursor = create_class_with_framework(
-            AgnosticGridOutCursor, self._framework, self.__module__)
+            AgnosticGridOutCursor, self._framework, self.__module__
+        )
 
         return grid_out_cursor(cursor, self.collection)
+
+
+def _hash_gridout(gridout):
+    """Compute the effective hash of a GridOut object for use with an Etag header.
+
+    Create a FIPS-compliant Etag HTTP header hash using sha256
+    We use the _id + length + upload_date as a proxy for
+    uniqueness to avoid reading the entire file.
+    """
+    grid_hash = hashlib.sha256(str(gridout._id).encode("utf8"))
+    grid_hash.update(str(gridout.length).encode("utf8"))
+    grid_hash.update(str(gridout.upload_date).encode("utf8"))
+    return grid_hash.hexdigest()
