@@ -18,23 +18,23 @@ import asyncio
 import sys
 import traceback
 import unittest
+from test.asyncio_tests import AsyncIOTestCase, asyncio_test
+from test.test_environment import env
+from test.utils import ignore_deprecations
 
-import bson
+import pymongo
 from bson import CodecOptions
 from bson.binary import JAVA_LEGACY
 from pymongo import ReadPreference, WriteConcern
+from pymongo.encryption import Algorithm, QueryType
 from pymongo.errors import BulkWriteError, DuplicateKeyError, OperationFailure
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import Secondary
 
-from motor import motor_asyncio
-from motor.motor_asyncio import (AsyncIOMotorCollection,
-                                 AsyncIOMotorCommandCursor)
-import test
-from test.asyncio_tests import (asyncio_test,
-                                AsyncIOTestCase,
-                                skip_if_mongos)
-from test.utils import ignore_deprecations
+from motor.motor_asyncio import AsyncIOMotorClientEncryption, AsyncIOMotorCollection
+
+if pymongo.version_tuple >= (4, 4, 0):
+    from pymongo.encryption_options import RangeOpts
 
 
 class TestAsyncIOCollection(AsyncIOTestCase):
@@ -42,39 +42,34 @@ class TestAsyncIOCollection(AsyncIOTestCase):
     async def test_collection(self):
         # Test that we can create a collection directly, not just from
         # database accessors.
-        collection = AsyncIOMotorCollection(self.db, 'test_collection')
+        collection = AsyncIOMotorCollection(self.db, "test_collection")
 
         # Make sure we got the right collection and it can do an operation
-        self.assertEqual('test_collection', collection.name)
+        self.assertEqual("test_collection", collection.name)
         await collection.delete_many({})
-        await collection.insert_one({'_id': 1})
-        doc = await collection.find_one({'_id': 1})
-        self.assertEqual(1, doc['_id'])
+        await collection.insert_one({"_id": 1})
+        doc = await collection.find_one({"_id": 1})
+        self.assertEqual(1, doc["_id"])
 
         # If you pass kwargs to PyMongo's Collection(), it calls
         # db.create_collection(). Motor can't do I/O in a constructor
         # so this is prohibited.
         self.assertRaises(
-            TypeError,
-            AsyncIOMotorCollection,
-            self.db,
-            'test_collection',
-            capped=True)
+            TypeError, AsyncIOMotorCollection, self.db, "test_collection", capped=True
+        )
 
     @asyncio_test
     async def test_dotted_collection_name(self):
         # Ensure that remove, insert, and find work on collections with dots
         # in their names.
-        for coll in (
-                self.db.foo.bar,
-                self.db.foo.bar.baz):
+        for coll in (self.db.foo.bar, self.db.foo.bar.baz):
             await coll.delete_many({})
-            result = await coll.insert_one({'_id': 'xyzzy'})
-            self.assertEqual('xyzzy', result.inserted_id)
-            result = await coll.find_one({'_id': 'xyzzy'})
-            self.assertEqual(result['_id'], 'xyzzy')
+            result = await coll.insert_one({"_id": "xyzzy"})
+            self.assertEqual("xyzzy", result.inserted_id)
+            result = await coll.find_one({"_id": "xyzzy"})
+            self.assertEqual(result["_id"], "xyzzy")
             await coll.delete_many({})
-            resp = await coll.find_one({'_id': 'xyzzy'})
+            resp = await coll.find_one({"_id": "xyzzy"})
             self.assertEqual(None, resp)
 
     def test_call(self):
@@ -82,16 +77,15 @@ class TestAsyncIOCollection(AsyncIOTestCase):
         try:
             self.db.foo()
         except TypeError as e:
-            self.assertTrue('no such method exists' in str(e))
+            self.assertTrue("no such method exists" in str(e))
         else:
-            self.fail('Expected TypeError')
+            self.fail("Expected TypeError")
 
     @ignore_deprecations
     @asyncio_test
     async def test_update(self):
-        await self.collection.insert_one({'_id': 1})
-        result = await self.collection.update_one(
-            {'_id': 1}, {'$set': {'foo': 'bar'}})
+        await self.collection.insert_one({"_id": 1})
+        result = await self.collection.update_one({"_id": 1}, {"$set": {"foo": "bar"}})
 
         self.assertIsNone(result.upserted_id)
         self.assertEqual(1, result.modified_count)
@@ -101,12 +95,12 @@ class TestAsyncIOCollection(AsyncIOTestCase):
     async def test_update_bad(self):
         # Violate a unique index, make sure we handle error well
         coll = self.db.unique_collection
-        await coll.create_index('s', unique=True)
+        await coll.create_index("s", unique=True)
 
         try:
-            await coll.insert_many([{'s': 1}, {'s': 2}])
+            await coll.insert_many([{"s": 1}, {"s": 2}])
             with self.assertRaises(DuplicateKeyError):
-                await coll.update_one({'s': 2}, {'$set': {'s': 1}})
+                await coll.update_one({"s": 2}, {"$set": {"s": 1}})
 
         finally:
             await coll.drop()
@@ -114,52 +108,46 @@ class TestAsyncIOCollection(AsyncIOTestCase):
     @asyncio_test
     async def test_insert_one(self):
         collection = self.collection
-        result = await collection.insert_one({'_id': 201})
+        result = await collection.insert_one({"_id": 201})
         self.assertEqual(201, result.inserted_id)
 
     @ignore_deprecations
     @asyncio_test
     async def test_insert_many_one_bad(self):
         collection = self.collection
-        await collection.insert_one({'_id': 2})
+        await collection.insert_one({"_id": 2})
 
         # Violate a unique index in one of many updates, handle error.
         with self.assertRaises(BulkWriteError):
-            await collection.insert_many([
-                {'_id': 1},
-                {'_id': 2},  # Already exists
-                {'_id': 3}])
+            await collection.insert_many([{"_id": 1}, {"_id": 2}, {"_id": 3}])  # Already exists
 
         # First insert should have succeeded, but not second or third.
-        self.assertEqual(
-            set([1, 2]),
-            set((await collection.distinct('_id'))))
+        self.assertEqual(set([1, 2]), set(await collection.distinct("_id")))
 
     @asyncio_test
     async def test_delete_one(self):
         # Remove a document twice, check that we get a success responses
         # and n = 0 for the second time.
-        await self.collection.insert_one({'_id': 1})
-        result = await self.collection.delete_one({'_id': 1})
+        await self.collection.insert_one({"_id": 1})
+        result = await self.collection.delete_one({"_id": 1})
 
         # First time we remove, n = 1
-        self.assertEqual(1, result.raw_result['n'])
-        self.assertEqual(1, result.raw_result['ok'])
-        self.assertEqual(None, result.raw_result.get('err'))
+        self.assertEqual(1, result.raw_result["n"])
+        self.assertEqual(1, result.raw_result["ok"])
+        self.assertEqual(None, result.raw_result.get("err"))
 
-        result = await self.collection.delete_one({'_id': 1})
+        result = await self.collection.delete_one({"_id": 1})
 
         # Second time, document is already gone, n = 0
-        self.assertEqual(0, result.raw_result['n'])
-        self.assertEqual(1, result.raw_result['ok'])
-        self.assertEqual(None, result.raw_result.get('err'))
+        self.assertEqual(0, result.raw_result["n"])
+        self.assertEqual(1, result.raw_result["ok"])
+        self.assertEqual(None, result.raw_result.get("err"))
 
     @ignore_deprecations
     @asyncio_test
     async def test_unacknowledged_insert(self):
         coll = self.db.test_unacknowledged_insert
-        await coll.with_options(write_concern=WriteConcern(0)).insert_one(
-            {'_id': 1})
+        await coll.with_options(write_concern=WriteConcern(0)).insert_one({"_id": 1})
 
         # The insert is eventually executed.
         while not (await coll.count_documents({})):
@@ -170,81 +158,33 @@ class TestAsyncIOCollection(AsyncIOTestCase):
     async def test_unacknowledged_update(self):
         coll = self.collection
 
-        await coll.insert_one({'_id': 1})
+        await coll.insert_one({"_id": 1})
         await coll.with_options(write_concern=WriteConcern(0)).update_one(
-            {'_id': 1}, {'$set': {'a': 1}})
+            {"_id": 1}, {"$set": {"a": 1}}
+        )
 
-        while not (await coll.find_one({'a': 1})):
+        while not (await coll.find_one({"a": 1})):
             await asyncio.sleep(0.1)
 
-    @asyncio_test
-    async def test_map_reduce(self):
-        # Count number of documents with even and odd _id
-        await self.make_test_data()
-        expected_result = [{'_id': 0, 'value': 100}, {'_id': 1, 'value': 100}]
-        map_fn = bson.Code('function map() { emit(this._id % 2, 1); }')
-        reduce_fn = bson.Code('''
-        function reduce(key, values) {
-            r = 0;
-            values.forEach(function(value) { r += value; });
-            return r;
-        }''')
-
-        await self.db.tmp_mr.drop()
-
-        # First do a standard mapreduce, should return AsyncIOMotorCollection
-        collection = self.collection
-        tmp_mr = await collection.map_reduce(map_fn, reduce_fn, 'tmp_mr')
-
-        self.assertTrue(
-            isinstance(tmp_mr, motor_asyncio.AsyncIOMotorCollection),
-            'map_reduce should return AsyncIOMotorCollection, not %s' % tmp_mr)
-
-        result = await tmp_mr.find().sort([('_id', 1)]).to_list(
-            length=1000)
-        self.assertEqual(expected_result, result)
-
-        # Standard mapreduce with full response
-        await self.db.tmp_mr.drop()
-        response = await collection.map_reduce(
-            map_fn, reduce_fn, 'tmp_mr', full_response=True)
-
-        self.assertTrue(
-            isinstance(response, dict),
-            'map_reduce should return dict, not %s' % response)
-
-        self.assertEqual('tmp_mr', response['result'])
-        result = await tmp_mr.find().sort([('_id', 1)]).to_list(
-            length=1000)
-        self.assertEqual(expected_result, result)
-
-        # Inline mapreduce
-        await self.db.tmp_mr.drop()
-        result = await collection.inline_map_reduce(
-            map_fn, reduce_fn)
-
-        result.sort(key=lambda doc: doc['_id'])
-        self.assertEqual(expected_result, result)
-
     @ignore_deprecations
     @asyncio_test
     async def test_indexes(self):
         test_collection = self.collection
 
         # Create an index
-        idx_name = await test_collection.create_index([('foo', 1)])
+        idx_name = await test_collection.create_index([("foo", 1)])
         index_info = await test_collection.index_information()
-        self.assertEqual([('foo', 1)], index_info[idx_name]['key'])
+        self.assertEqual([("foo", 1)], index_info[idx_name]["key"])
 
         # Don't test drop_index or drop_indexes -- Synchro tests them
 
     async def _make_test_data(self, n):
         await self.db.drop_collection("test")
-        await self.db.test.insert_many([{'_id': i} for i in range(n)])
+        await self.db.test.insert_many([{"_id": i} for i in range(n)])
         expected_sum = sum(range(n))
         return expected_sum
 
-    pipeline = [{'$project': {'_id': '$_id'}}]
+    pipeline = [{"$project": {"_id": "$_id"}}]
 
     @asyncio_test(timeout=30)
     async def test_aggregation_cursor(self):
@@ -256,8 +196,7 @@ class TestAsyncIOCollection(AsyncIOTestCase):
             expected_sum = await self._make_test_data(collection_size)
             cursor = db.test.aggregate(self.pipeline)
             docs = await cursor.to_list(collection_size)
-            self.assertEqual(expected_sum,
-                             sum(doc['_id'] for doc in docs))
+            self.assertEqual(expected_sum, sum(doc["_id"] for doc in docs))
 
     @asyncio_test
     async def test_aggregation_cursor_exc_info(self):
@@ -271,18 +210,25 @@ class TestAsyncIOCollection(AsyncIOTestCase):
             _, _, tb = sys.exc_info()
 
             # The call tree should include PyMongo code we ran on a thread.
-            formatted = '\n'.join(traceback.format_tb(tb))
-            self.assertTrue('_unpack_response' in formatted
-                            or '_check_command_response' in formatted)
+            formatted = "\n".join(traceback.format_tb(tb))
+            self.assertTrue(
+                "_unpack_response" in formatted or "_check_command_response" in formatted
+            )
+
+    @asyncio_test
+    async def test_aggregate_cursor_del(self):
+        cursor = self.db.test.aggregate(self.pipeline)
+        del cursor
+        cursor = self.db.test.aggregate(self.pipeline)
+        await cursor.close()
+        del cursor
 
     def test_with_options(self):
         coll = self.db.test
-        codec_options = CodecOptions(
-            tz_aware=True, uuid_representation=JAVA_LEGACY)
+        codec_options = CodecOptions(tz_aware=True, uuid_representation=JAVA_LEGACY)
 
         write_concern = WriteConcern(w=2, j=True)
-        coll2 = coll.with_options(
-            codec_options, ReadPreference.SECONDARY, write_concern)
+        coll2 = coll.with_options(codec_options, ReadPreference.SECONDARY, write_concern)
 
         self.assertTrue(isinstance(coll2, AsyncIOMotorCollection))
         self.assertEqual(codec_options, coll2.codec_options)
@@ -301,18 +247,18 @@ class TestAsyncIOCollection(AsyncIOTestCase):
         write_concern = WriteConcern(w=2, j=True)
         read_concern = ReadConcern("majority")
         read_preference = Secondary([{"dc": "sf"}])
-        codec_options = CodecOptions(
-            tz_aware=True, uuid_representation=JAVA_LEGACY)
+        codec_options = CodecOptions(tz_aware=True, uuid_representation=JAVA_LEGACY)
 
         coll1 = self.db.get_collection(
-            'test',
+            "test",
             write_concern=write_concern,
             read_concern=read_concern,
             read_preference=read_preference,
-            codec_options=codec_options)
+            codec_options=codec_options,
+        )
 
         coll2 = coll1.subcollection
-        coll3 = coll1['subcollection']
+        coll3 = coll1["subcollection"]
 
         for c in [coll1, coll2, coll3]:
             self.assertEqual(write_concern, c.write_concern)
@@ -320,6 +266,76 @@ class TestAsyncIOCollection(AsyncIOTestCase):
             self.assertEqual(read_preference, c.read_preference)
             self.assertEqual(codec_options, c.codec_options)
 
+    @env.require_version_min(7, 0, -1, -1)
+    @env.require_no_standalone
+    @asyncio_test
+    async def test_async_create_encrypted_collection(self):
+        c = self.collection
+        KMS_PROVIDERS = {"local": {"key": b"\x00" * 96}}
+        self.cx.drop_database("db")
+        async with AsyncIOMotorClientEncryption(
+            KMS_PROVIDERS, "keyvault.datakeys", c, CodecOptions()
+        ) as client_encryption:
+            coll, ef = await client_encryption.create_encrypted_collection(
+                database=self.db,
+                name="testing1",
+                encrypted_fields={"fields": [{"path": "ssn", "bsonType": "string", "keyId": None}]},
+                kms_provider="local",
+            )
+            with self.assertRaises(pymongo.errors.WriteError) as exc:
+                await coll.insert_one({"ssn": "123-45-6789"})
+            self.assertEqual(exc.exception.code, 121)
+            await self.db.drop_collection("testing1", encrypted_fields=ef)
 
-if __name__ == '__main__':
+    @asyncio_test
+    async def test_async_encrypt_expression(self):
+        c = self.collection
+        KMS_PROVIDERS = {"local": {"key": b"\x00" * 96}}
+        self.cx.drop_database("db")
+        async with AsyncIOMotorClientEncryption(
+            KMS_PROVIDERS, "keyvault.datakeys", c, CodecOptions()
+        ) as client_encryption:
+            data_key = await client_encryption.create_data_key(
+                "local", key_alt_names=["pymongo_encryption_example_1"]
+            )
+            name = "DoubleNoPrecision"
+            range_opts = RangeOpts(sparsity=1)
+            for i in [6.0, 30.0, 200.0]:
+                insert_payload = await client_encryption.encrypt(
+                    float(i),
+                    key_id=data_key,
+                    algorithm=Algorithm.RANGEPREVIEW,
+                    contention_factor=0,
+                    range_opts=range_opts,
+                )
+                self.collection.insert_one(
+                    {
+                        f"encrypted{name}": insert_payload,
+                    }
+                )
+                self.assertEqual(await client_encryption.decrypt(insert_payload), i)
+
+            find_payload = await client_encryption.encrypt_expression(
+                expression={
+                    "$and": [
+                        {f"encrypted{name}": {"$gte": 6.0}},
+                        {f"encrypted{name}": {"$lte": 200.0}},
+                    ]
+                },
+                key_id=data_key,
+                algorithm=Algorithm.RANGEPREVIEW,
+                query_type=QueryType.RANGEPREVIEW,
+                contention_factor=0,
+                range_opts=range_opts,
+            )
+
+            sorted_find = sorted(
+                await self.collection.explicit_encryption.find(find_payload).to_list(3),
+                key=lambda x: x["_id"],
+            )
+            for elem, expected in zip(sorted_find, [6.0, 30.0, 200.0]):
+                self.assertEqual(elem[f"encrypted{name}"], expected)
+
+
+if __name__ == "__main__":
     unittest.main()
