@@ -19,25 +19,33 @@ import traceback
 import unittest
 import warnings
 from functools import partial
+from test import SkipTest, env
+from test.tornado_tests import (
+    MotorMockServerTest,
+    MotorTest,
+    get_command_line,
+    server_is_mongos,
+)
+from test.utils import (
+    TestListener,
+    get_async_test_timeout,
+    get_primary_pool,
+    one,
+    safe_get,
+    wait_until,
+)
 
 import bson
 import pymongo
+from pymongo import CursorType
+from pymongo.collation import Collation
+from pymongo.errors import ExecutionTimeout, InvalidOperation, OperationFailure
 from tornado import gen
 from tornado.concurrent import Future
 from tornado.testing import gen_test
-from pymongo import CursorType
-from pymongo.collation import Collation
-from pymongo.errors import InvalidOperation, ExecutionTimeout
-from pymongo.errors import OperationFailure
 
 import motor
 import motor.motor_tornado
-from test import SkipTest, env
-from test.tornado_tests import (get_command_line,
-                                MotorTest,
-                                MotorMockServerTest,
-                                server_is_mongos)
-from test.utils import one, safe_get, get_primary_pool, TestListener
 
 
 class MotorCursorTest(MotorMockServerTest):
@@ -50,22 +58,20 @@ class MotorCursorTest(MotorMockServerTest):
     async def test_count(self):
         await self.make_test_data()
         coll = self.collection
-        self.assertEqual(100,
-                         (await coll.count_documents({'_id': {'$gt': 99}})))
+        self.assertEqual(100, (await coll.count_documents({"_id": {"$gt": 99}})))
 
     @gen_test
     async def test_fetch_next(self):
         await self.make_test_data()
         coll = self.collection
         # 200 results, only including _id field, sorted by _id
-        cursor = coll.find({}, {'_id': 1}).sort(
-            [('_id', pymongo.ASCENDING)]).batch_size(75)
+        cursor = coll.find({}, {"_id": 1}).sort([("_id", pymongo.ASCENDING)]).batch_size(75)
 
         self.assertEqual(None, cursor.cursor_id)
         self.assertEqual(None, cursor.next_object())  # Haven't fetched yet
         i = 0
-        while (await cursor.fetch_next):
-            self.assertEqual({'_id': i}, cursor.next_object())
+        while await cursor.fetch_next:
+            self.assertEqual({"_id": i}, cursor.next_object())
             i += 1
             # With batch_size 75 and 200 results, cursor should be exhausted on
             # the server by third fetch
@@ -81,8 +87,8 @@ class MotorCursorTest(MotorMockServerTest):
 
     @gen_test
     async def test_fetch_next_delete(self):
-        if 'PyPy' in sys.version:
-            raise SkipTest('PyPy')
+        if "PyPy" in sys.version:
+            raise SkipTest("PyPy")
 
         client, server = self.client_server(auto_ismaster=True)
         cursor = client.test.coll.find()
@@ -90,10 +96,7 @@ class MotorCursorTest(MotorMockServerTest):
         # With Tornado, simply accessing fetch_next starts the fetch.
         cursor.fetch_next
         request = await self.run_thread(server.receives, "find", "coll")
-        request.replies({"cursor": {
-            "id": 123,
-            "ns": "db.coll",
-            "firstBatch": [{"_id": 1}]}})
+        request.replies({"cursor": {"id": 123, "ns": "db.coll", "firstBatch": [{"_id": 1}]}})
 
         # Decref'ing the cursor eventually closes it on the server.
         del cursor
@@ -105,7 +108,7 @@ class MotorCursorTest(MotorMockServerTest):
     async def test_fetch_next_without_results(self):
         coll = self.collection
         # Nothing matches this query
-        cursor = coll.find({'foo': 'bar'})
+        cursor = coll.find({"foo": "bar"})
         self.assertEqual(None, cursor.next_object())
         self.assertEqual(False, (await cursor.fetch_next))
         self.assertEqual(None, cursor.next_object())
@@ -129,26 +132,31 @@ class MotorCursorTest(MotorMockServerTest):
     @gen_test
     async def test_fetch_next_exception(self):
         coll = self.collection
-        cursor = coll.find()
-        cursor.delegate._Cursor__id = 1234  # Not valid on server
+        await coll.insert_many([{} for _ in range(10)])
+        cursor = coll.find(batch_size=2)
+        await cursor.fetch_next
+        self.assertTrue(cursor.next_object())
+
+        # Not valid on server, causes CursorNotFound.
+        cursor.delegate._Cursor__id = bson.int64.Int64(1234)
 
         with self.assertRaises(OperationFailure):
             await cursor.fetch_next
-
-        # Avoid the cursor trying to close itself when it goes out of scope
-        cursor.delegate._Cursor__id = None
+            self.assertTrue(cursor.next_object())
+            await cursor.fetch_next
+            self.assertTrue(cursor.next_object())
 
     def test_each_callback(self):
         cursor = self.collection.find()
-        self.assertRaises(TypeError, cursor.each, callback='foo')
+        self.assertRaises(TypeError, cursor.each, callback="foo")
         self.assertRaises(TypeError, cursor.each, callback=None)
         self.assertRaises(TypeError, cursor.each)  # No callback.
 
     @gen_test(timeout=30)
     async def test_each(self):
         await self.make_test_data()
-        cursor = self.collection.find({}, {'_id': 1})
-        cursor.sort([('_id', pymongo.ASCENDING)])
+        cursor = self.collection.find({}, {"_id": 1})
+        cursor.sort([("_id", pymongo.ASCENDING)])
         future = Future()
         results = []
 
@@ -164,7 +172,7 @@ class MotorCursorTest(MotorMockServerTest):
 
         cursor.each(callback)
         await future
-        expected = [{'_id': i} for i in range(200)]
+        expected = [{"_id": i} for i in range(200)]
         self.assertEqual(expected, results)
 
     @gen_test
@@ -177,17 +185,17 @@ class MotorCursorTest(MotorMockServerTest):
             await cursor.to_list(-1)
 
         with self.assertRaises(TypeError):
-            await cursor.to_list('foo')
+            await cursor.to_list("foo")
 
     @gen_test
     async def test_to_list_with_length(self):
         await self.make_test_data()
         coll = self.collection
-        cursor = coll.find().sort('_id')
+        cursor = coll.find().sort("_id")
         self.assertEqual([], (await cursor.to_list(0)))
 
         def expected(start, stop):
-            return [{'_id': i} for i in range(start, stop)]
+            return [{"_id": i} for i in range(start, stop)]
 
         self.assertEqual(expected(0, 10), (await cursor.to_list(10)))
         self.assertEqual(expected(10, 100), (await cursor.to_list(90)))
@@ -209,9 +217,6 @@ class MotorCursorTest(MotorMockServerTest):
 
     @gen_test
     async def test_to_list_exc_info(self):
-        if sys.version_info < (3,):
-            raise SkipTest("Requires Python 3")
-
         await self.make_test_data()
         coll = self.collection
         cursor = coll.find()
@@ -223,9 +228,10 @@ class MotorCursorTest(MotorMockServerTest):
             _, _, tb = sys.exc_info()
 
             # The call tree should include PyMongo code we ran on a thread.
-            formatted = '\n'.join(traceback.format_tb(tb))
-            self.assertTrue('_unpack_response' in formatted
-                            or '_check_command_response' in formatted)
+            formatted = "\n".join(traceback.format_tb(tb))
+            self.assertTrue(
+                "_unpack_response" in formatted or "_check_command_response" in formatted
+            )
 
     @gen_test
     async def test_to_list_with_length_of_none(self):
@@ -249,10 +255,12 @@ class MotorCursorTest(MotorMockServerTest):
     @gen_test
     async def test_to_list_with_chained_collation(self):
         await self.make_test_data()
-        cursor = self.collection.find({}, {'_id': 1}) \
-            .sort([('_id', pymongo.ASCENDING)]) \
+        cursor = (
+            self.collection.find({}, {"_id": 1})
+            .sort([("_id", pymongo.ASCENDING)])
             .collation(Collation("en"))
-        expected = [{'_id': i} for i in range(200)]
+        )
+        expected = [{"_id": i} for i in range(200)]
         result = await cursor.to_list(length=1000)
         self.assertEqual(expected, result)
 
@@ -265,22 +273,21 @@ class MotorCursorTest(MotorMockServerTest):
         # With Tornado, simply accessing fetch_next starts the fetch.
         fetch_next = cursor.fetch_next
         request = await self.run_thread(server.receives, "find", "coll")
-        request.replies({"cursor": {
-            "id": 123,
-            "ns": "db.coll",
-            "firstBatch": [{"_id": 1}]}})
+        request.replies({"cursor": {"id": 123, "ns": "db.coll", "firstBatch": [{"_id": 1}]}})
 
-        self.assertTrue((await fetch_next))
+        self.assertTrue(await fetch_next)
 
-        await cursor.close()
-        request = await self.run_thread(server.receives, "killCursors", "coll")
-        request.ok()
+        async def mock_kill_cursors():
+            request = await self.run_thread(server.receives, "killCursors", "coll")
+            request.ok()
+
+        await gen.multi([cursor.close(), mock_kill_cursors()])
 
         # Cursor reports it's alive because it has buffered data, even though
         # it's killed on the server.
         self.assertTrue(cursor.alive)
-        self.assertEqual({'_id': 1}, cursor.next_object())
-        self.assertFalse((await cursor.fetch_next))
+        self.assertEqual({"_id": 1}, cursor.next_object())
+        self.assertFalse(await cursor.fetch_next)
         self.assertFalse(cursor.alive)
 
     @gen_test
@@ -314,7 +321,6 @@ class MotorCursorTest(MotorMockServerTest):
             if error:
                 future.set_exception(error)
             elif result:
-                pass
                 results.append(result)
             else:
                 # Complete
@@ -331,26 +337,26 @@ class MotorCursorTest(MotorMockServerTest):
         cursor = self.collection.find().limit(2)
 
         count = 0
-        while (await cursor.fetch_next):
+        while await cursor.fetch_next:
             cursor.next_object()
             count += 1
         self.assertEqual(2, count)
 
         cursor.rewind()
         count = 0
-        while (await cursor.fetch_next):
+        while await cursor.fetch_next:
             cursor.next_object()
             count += 1
         self.assertEqual(2, count)
 
         cursor.rewind()
         count = 0
-        while (await cursor.fetch_next):
+        while await cursor.fetch_next:
             cursor.next_object()
             break
 
         cursor.rewind()
-        while (await cursor.fetch_next):
+        while await cursor.fetch_next:
             cursor.next_object()
             count += 1
 
@@ -359,10 +365,7 @@ class MotorCursorTest(MotorMockServerTest):
 
     @gen_test
     async def test_cursor_del(self):
-        if sys.version_info < (3, 4):
-            raise SkipTest("requires Python 3.4")
-
-        if 'PyPy' in sys.version:
+        if "PyPy" in sys.version:
             raise SkipTest("PyPy")
 
         client, server = self.client_server(auto_ismaster=True)
@@ -370,10 +373,7 @@ class MotorCursorTest(MotorMockServerTest):
 
         future = cursor.fetch_next
         request = await self.run_thread(server.receives, "find", "coll")
-        request.replies({"cursor": {
-            "id": 123,
-            "ns": "db.coll",
-            "firstBatch": [{"_id": 1}]}})
+        request.replies({"cursor": {"id": 123, "ns": "db.coll", "firstBatch": [{"_id": 1}]}})
         await future  # Complete the first fetch.
 
         # Dereference the cursor.
@@ -387,12 +387,8 @@ class MotorCursorTest(MotorMockServerTest):
 
     @gen_test
     async def test_exhaust(self):
-        if sys.version_info < (3, 4):
-            raise SkipTest("requires Python 3.4")
-
-        if (await server_is_mongos(self.cx)):
-            self.assertRaises(InvalidOperation,
-                              self.db.test.find, cursor_type=CursorType.EXHAUST)
+        if await server_is_mongos(self.cx):
+            self.assertRaises(InvalidOperation, self.db.test.find, cursor_type=CursorType.EXHAUST)
             return
 
         cur = self.db.test.find(cursor_type=CursorType.EXHAUST)
@@ -411,48 +407,54 @@ class MotorCursorTest(MotorMockServerTest):
         client = self.motor_client(maxPoolSize=1)
         # Ensure a pool.
         await client.db.collection.find_one()
-        socks = get_primary_pool(client).sockets
+        pool = get_primary_pool(client)
+        conns = pool.conns
 
         # Make sure the socket is returned after exhaustion.
         cur = client[self.db.name].test.find(cursor_type=CursorType.EXHAUST)
         has_next = await cur.fetch_next
         self.assertTrue(has_next)
-        self.assertEqual(0, len(socks))
+        self.assertEqual(0, len(conns))
 
-        while (await cur.fetch_next):
+        while await cur.fetch_next:
             cur.next_object()
 
-        self.assertEqual(1, len(socks))
+        self.assertEqual(1, len(conns))
 
         # Same as previous but with to_list instead of next_object.
-        docs = await client[self.db.name].test.find(
-            cursor_type=CursorType.EXHAUST).to_list(None)
-        self.assertEqual(1, len(socks))
-        self.assertEqual(
-            (await self.db.test.count_documents({})),
-            len(docs))
+        docs = await client[self.db.name].test.find(cursor_type=CursorType.EXHAUST).to_list(None)
+        self.assertEqual(1, len(conns))
+        self.assertEqual((await self.db.test.count_documents({})), len(docs))
 
         # If the Cursor instance is discarded before being
         # completely iterated we have to close and
         # discard the socket.
-        sock = one(socks)
-        cur = client[self.db.name].test.find(
-            cursor_type=CursorType.EXHAUST).batch_size(1)
-        has_next = await cur.fetch_next
-        self.assertTrue(has_next)
-        self.assertEqual(0, len(socks))
-        if 'PyPy' in sys.version:
+        conn = one(conns)
+        cur = client[self.db.name].test.find(cursor_type=CursorType.EXHAUST).batch_size(1)
+        await cur.fetch_next
+        self.assertTrue(cur.next_object())
+        # Run at least one getMore to initiate the OP_MSG exhaust protocol.
+        if env.version.at_least(4, 2):
+            await cur.fetch_next
+            self.assertTrue(cur.next_object())
+        self.assertEqual(0, len(conns))
+        if "PyPy" in sys.version:
             # Don't wait for GC or use gc.collect(), it's unreliable.
             await cur.close()
 
         del cur
 
-        await gen.sleep(0.1)
+        async def conn_closed():
+            return conn not in conns and conn.closed
+
+        await wait_until(
+            conn_closed, "close exhaust cursor socket", timeout=get_async_test_timeout()
+        )
 
         # The exhaust cursor's socket was discarded, although another may
         # already have been opened to send OP_KILLCURSORS.
-        self.assertNotIn(sock, socks)
-        self.assertTrue(sock.closed)
+        self.assertNotIn(conn, conns)
+        self.assertTrue(conn.closed)
 
     def test_iter(self):
         # Iteration should be prohibited.
@@ -480,39 +482,49 @@ class MotorCursorTest(MotorMockServerTest):
         cx = self.motor_client(event_listeners=[listener])
         c = cx.motor_test.collection
         await c.delete_many({})
-        await c.insert_many({'_id': i} for i in range(3))
+        await c.insert_many({"_id": i} for i in range(3))
 
         # Two ways of setting batchSize.
-        cursor0 = c.aggregate([{'$sort': {'_id': 1}}]).batch_size(2)
-        cursor1 = c.aggregate([{'$sort': {'_id': 1}}], batchSize=2)
+        cursor0 = c.aggregate([{"$sort": {"_id": 1}}]).batch_size(2)
+        cursor1 = c.aggregate([{"$sort": {"_id": 1}}], batchSize=2)
         for cursor in cursor0, cursor1:
             lst = []
-            while (await cursor.fetch_next):
+            while await cursor.fetch_next:
                 lst.append(cursor.next_object())
 
-            self.assertEqual(lst, [{'_id': 0}, {'_id': 1}, {'_id': 2}])
-            aggregate = listener.first_command_started('aggregate')
-            self.assertEqual(aggregate.command['cursor']['batchSize'], 2)
-            getMore = listener.first_command_started('getMore')
-            self.assertEqual(getMore.command['batchSize'], 2)
+            self.assertEqual(lst, [{"_id": 0}, {"_id": 1}, {"_id": 2}])
+            aggregate = listener.first_command_started("aggregate")
+            self.assertEqual(aggregate.command["cursor"]["batchSize"], 2)
+            getMore = listener.first_command_started("getMore")
+            self.assertEqual(getMore.command["batchSize"], 2)
 
     @gen_test
     async def test_raw_batches(self):
         c = self.collection
         await c.delete_many({})
-        await c.insert_many({'_id': i} for i in range(4))
+        await c.insert_many({"_id": i} for i in range(4))
 
         find = partial(c.find_raw_batches, {})
-        agg = partial(c.aggregate_raw_batches, [{'$sort': {'_id': 1}}])
+        agg = partial(c.aggregate_raw_batches, [{"$sort": {"_id": 1}}])
 
         for method in find, agg:
             cursor = method().batch_size(2)
             await cursor.fetch_next
             batch = cursor.next_object()
-            self.assertEqual([{'_id': 0}, {'_id': 1}], bson.decode_all(batch))
+            self.assertEqual([{"_id": 0}, {"_id": 1}], bson.decode_all(batch))
 
             lst = await method().batch_size(2).to_list(length=1)
-            self.assertEqual([{'_id': 0}, {'_id': 1}], bson.decode_all(lst[0]))
+            self.assertEqual([{"_id": 0}, {"_id": 1}], bson.decode_all(lst[0]))
+
+    @gen_test
+    async def test_generate_keys(self):
+        c = self.collection
+        KMS_PROVIDERS = {"local": {"key": b"\x00" * 96}}
+
+        async with motor.MotorClientEncryption(
+            KMS_PROVIDERS, "keyvault.datakeys", c, bson.codec_options.CodecOptions()
+        ) as client_encryption:
+            self.assertIsInstance(await client_encryption.get_keys(), motor.MotorCursor)
 
 
 class MotorCursorMaxTimeMSTest(MotorTest):
@@ -525,23 +537,19 @@ class MotorCursorMaxTimeMSTest(MotorTest):
         super().tearDown()
 
     async def maybe_skip(self):
-        if (await server_is_mongos(self.cx)):
+        if await server_is_mongos(self.cx):
             raise SkipTest("mongos has no maxTimeAlwaysTimeOut fail point")
 
         cmdline = await get_command_line(self.cx)
-        if '1' != safe_get(cmdline, 'parsed.setParameter.enableTestCommands'):
-            if 'enableTestCommands=1' not in cmdline['argv']:
+        if "1" != safe_get(cmdline, "parsed.setParameter.enableTestCommands"):
+            if "enableTestCommands=1" not in cmdline["argv"]:
                 raise SkipTest("testing maxTimeMS requires failpoints")
 
     async def enable_timeout(self):
-        await self.cx.admin.command("configureFailPoint",
-                                    "maxTimeAlwaysTimeOut",
-                                    mode="alwaysOn")
+        await self.cx.admin.command("configureFailPoint", "maxTimeAlwaysTimeOut", mode="alwaysOn")
 
     async def disable_timeout(self):
-        await self.cx.admin.command("configureFailPoint",
-                                    "maxTimeAlwaysTimeOut",
-                                    mode="off")
+        await self.cx.admin.command("configureFailPoint", "maxTimeAlwaysTimeOut", mode="off")
 
     @gen_test
     async def test_max_time_ms_query(self):
@@ -571,7 +579,7 @@ class MotorCursorMaxTimeMSTest(MotorTest):
             # Test getmore timeout.
             await self.enable_timeout()
             with self.assertRaises(ExecutionTimeout):
-                while (await cursor.fetch_next):
+                while await cursor.fetch_next:
                     cursor.next_object()
 
             await cursor.close()
@@ -643,5 +651,5 @@ class MotorCursorMaxTimeMSTest(MotorTest):
             await self.collection.delete_many({})
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
